import bpy
from bpy.app.handlers import persistent

# Track original resolution per scene
_original_render_resolution = {}

def register_camera_resolution_props():
    bpy.types.Camera.per_res_enable = bpy.props.BoolProperty(
        name="Use Custom Resolution",
        description="Enable custom resolution for this camera",
        default=False
    )
    bpy.types.Camera.per_res_x = bpy.props.IntProperty(
        name="Resolution X",
        default=1920,
        min=1
    )
    bpy.types.Camera.per_res_y = bpy.props.IntProperty(
        name="Resolution Y",
        default=1080,
        min=1
    )

def unregister_camera_resolution_props():
    del bpy.types.Camera.per_res_enable
    del bpy.types.Camera.per_res_x
    del bpy.types.Camera.per_res_y

@persistent
def apply_per_camera_resolution(scene):
    cam = scene.camera
    if not cam or cam.type != 'CAMERA':
        return

    cam_data = cam.data
    if getattr(cam_data, "per_res_enable", False):
        # Save original render resolution
        if scene.name not in _original_render_resolution:
            _original_render_resolution[scene.name] = (
                scene.render.resolution_x,
                scene.render.resolution_y,
                scene.render.resolution_percentage
            )
        # Apply camera-specific resolution
        scene.render.resolution_x = cam_data.per_res_x
        scene.render.resolution_y = cam_data.per_res_y
        scene.render.resolution_percentage = 100
    else:
        # Restore original resolution if previously saved
        if scene.name in _original_render_resolution:
            x, y, scale = _original_render_resolution.pop(scene.name)
            scene.render.resolution_x = x
            scene.render.resolution_y = y
            scene.render.resolution_percentage = scale

def get_next_camera_name():
    index = 1
    while f"Camera2View_{index}" in bpy.data.cameras:
        index += 1
    return f"Camera2View_{index}"

def get_sorted_camera_objects():
    return sorted(
        [obj for obj in bpy.data.objects if obj.type == 'CAMERA' and obj.name.startswith("Camera2View_")],
        key=lambda obj: int(obj.name.split("_")[-1])
    )
